@@ -19,15 +19,23 @@
 #'          to allow easy output for multiple groups.
 #'          
 #'          The input should include either \code{formula} and \code{data};
-#'              or \code{data}, \code{var}, and \code{group}. (See examples). 
+#'              or \code{data}, \code{var}, and \code{group}. (See examples).
+#'              
+#'          Results for ungrouped (one-sample) data can be obtained by either
+#'          setting the right side of the formula to 1, e.g.  y ~ 1, or by
+#'          setting \code{group=NULL}.  
 #'          
 #' @note    The parsing of the formula is simplistic. The first variable on the
 #'          left side is used as the measurement variable.  The variables on the
 #'          right side are used for the grouping variables.          
 #'          
-#'        Results for ungrouped (one-sample) data can be obtained by either
-#'          setting the right side of the formula to 1, e.g.  y ~ 1, or by
-#'          setting \code{group=NULL}.                
+#'          It is recommended to remove \code{NA} values before using this
+#'          function.  At the time of writing, \code{NA} values will cause the
+#'          function to fail.
+#'          
+#'          At the time of writing, the \code{ci.type="boot"} option
+#'          produces \code{NA} results. This is a result from the
+#'          \code{DescTools::HuberM} function.            
 #'              
 #' @author Salvatore Mangiafico, \email{mangiafico@njaes.rutgers.edu}
 #' @references \url{http://rcompanion.org/rcompanion/d_08a.html}
@@ -67,7 +75,8 @@ groupwiseHuber =
     ddply(.data=data,
           .variables=group, var,
           .fun=function(x, idx){
-               sum(!is.na(x[,idx]))})
+               length(x[,idx])})
+  
   funny = function(x, idx){HuberM(x[,idx], 
                            conf.level=conf.level, ci.type=ci.type, ...)}
   D2=
